/*****************************************************************************/

/// \file       main.c
/// \author     Frank Redlich
/// \version    1.01
/// \date       22.02.2020
/// \brief      Digital Decoder fr Carrera
/// \details    Projekt:    Carrera_Decoder<br>
///             Prozessor:  Microchip PIC16F1824 <br>
///             Hardware:   Rev2a

/******************************************************************************/

/*
 * neu an 1.01
 * - Kombileuchte oder separate Rck- und Bremsleuchten 
 */

#include <xc.h>
#include "version.h"
#include "typedef.h"
#include "defines.h"
#include "variable.h"
#include "ports.h"
#include "init.h"
#include "decoder.h"
#include "eeprom.h"
#include "delays.h"

 

#pragma config FCMEN = OFF              // Fail-Safe Clock Monitor disabled
#pragma config IESO = OFF               // Internal/External Switchover mode is disabled
#pragma config CLKOUTEN = OFF           // CLKOUT function is disabled. I/O or oscillator function on the CLKOUT pin
//#pragma config BOREN = OFF              // Brown-out Reset disabled
#pragma config BOREN = ON               // Brown-out Reset enabled
#pragma config CPD = OFF                // Data memory code protection is disabled
#pragma config CP = OFF                 // Program memory code protection is disabled
#pragma config MCLRE = OFF              // MCLR/VPP pin function is digital input

#pragma config PWRTE = OFF              // power up timer disabled
//#pragma config PWRTE = ON               // power up timer enabled     (seltsamerweise geht der Reset hier ca 64ms schneller)

//#pragma config WDTE = OFF               // Watchdogtimer disabled
//#pragma config WDTE = ON                // Watchdogtimer always enabled
#pragma config WDTE = SWDTEN            // WDT controlled by the SWDTEN bit in the WDTCON register

#pragma config FOSC = INTOSC            // INTOSC oscillator: I/O function on CLKIN pin

#pragma config LVP = ON                 // Low-voltage programming enabled
//#pragma config BORV = HI                // Brown-out Reset Voltage (VBOR) set to 2.7 V
#pragma config BORV = LO                // Brown-out Reset Voltage (VBOR) set to 1.9 V
#pragma config STVREN = OFF             // Stack Overflow or Underflow will not cause a Reset

#pragma config PLLEN = ON               // 4xPLL enabled

#pragma config WRT = OFF                // Write protection off



void main()
{
    U8      Adresse;            
       
    
    OSCCONbits.IRCF = 0b1110;                                                   // interner Oszillator auf 8 MHz (4xPLL = 32 MHz)
    
    Initialisierung();                                                          // Ports, Timer etc. 
    
    Init_EEPROM_Defaultwerte();
    
    
    WDTCONbits.SWDTEN = 1;                                                      // Watchdog einschalten    
    CLRWDT();                                                                   // clear Watchdog     
    
    //--------------------------------------------------------------------------

    Licht_schalten();                                                           // je nach Wert im EEPROM Licht ein oder aus
    IR_LED_einschalten();                                                       // mit der Frequenz entsprechend der Car Adresse
   
    //--------------------------------------------------------------------------
    if ((EE_read(EE_ADR_PROGRAMMIERMODUS)) == JA)                               // Auto Adresse Programmiermodus JA/NEIN
        Adr_Programmier_timeout = PROGRAMMIER_TIMEOUT;                          // ca. 3 sec Programmiermglichkeit fr Adresse
    else
        Adr_Programmier_timeout = 0;
    
    
   
    
    while(1)
    {        
        if (Telegramm_empfangen == JA)                                          // alle 7,5 msec
        {            
            CLRWDT();                                                           // clear Watchdog 

            Telegramm_empfangen = NEIN;

            DataBit[0] = 'X';                                                   // Startbit ungltig machen fr erneutemn Empfang
            Timer_Counter++;                                                    // allgemeiner Timer 7,5 ms
            
            //...................................................
            
            if (Licht_Sperr_Timer > 0)                                          // auf Null zhlen
                Licht_Sperr_Timer--;                                            // jetzt darf Licht geschaltet werden

            eigene_Adresse = (U8)(EE_read(EE_ADR_AUTO_ADRESSE) & 0x07);         // wird in dieser Routine mehrfach bentigt  
                    
            switch (Telegrammtyp) 
            {
                /* hat sich nicht bewhrt, verursacht Aussetzer
                case 8:     //..................................................   Aktivdatenwort
                    
                    if (Auto.Eigene_Adresse < 6)                                // nur normale Autos, kein Ghostcar
                    {
                        if (DataBit[Auto.Eigene_Adresse + 1] == 0)              // wenn der eigene Regler nicht gedrckt
                            CCPR1L = 0;                                         // Motor ausschalten
                    }                                         
                    break;
                */ 

                case REGLERDATENWORT: //........................................  Regler oder Ghostcar Datenwort
                    
                    Reglerdatenwort_decodieren();

                    if (
                            (Ghostcar_Doppelklick_timeout == 0) &&              // nur wenn nicht auf Doppelklick fr Ghostcar gewartet wird
                            (CU_Daten.Adresse < 6)
                        )
                        Count_Spurwechseltaste(CU_Daten.Adresse);

                    if (CU_Daten.Adresse == eigene_Adresse)
                    {
                        Motor_und_Bremse_ansteuern(CU_Daten.Reglerwert);        // Motor in Abhngigkeit der Reglerstellung
                        if (Tank_leer == JA)                                    // wenn Tank leer dann blinken ==> kommt aus einem Programmierdatenwort
                            Tank_leer_Blinken = JA;
                            
                        Eigene_Spurwechseltaste_auswerten();                    // Schalten von Licht bzw. Wechsel in Ghostcarprogrammierung

                        if (Adr_warte_auf_Reset_timeout > 0)                    // wenn 2x Spurwechseltaste gedrckt gedrckt aber noch nicht angehoben
                        {
                            if (CU_Daten.Reglerwert > 0)                        // und wenn das Auto fhrt
                                Adr_warte_auf_Reset_timeout = 1;                // dann beim nchstenTimerclick den Programmiermodus sofort verlassen
                        }
                    }

                    if (eigene_Adresse == GHOSTCAR_ADRESSE)                     // Ghostcarbetrieb
                    {
                        v_Ghostcar = (U8)(EE_read(EE_ADR_GHOST_GESCHWINDIGKEIT) & 0x0F);
                                
                        if (CU_Daten.Adresse == PACE_UND_GHOSTCAR_DW_ADRESSE)
                        {
                            CU_Daten.KFR = DataBit[4];                          // Keine Freigabe Bit
                            CU_Daten.FR = DataBit[6];                           // Freigabe Bit
                            if ((CU_Daten.KFR == 0) && (CU_Daten.FR == 1))
                                Motor_und_Bremse_ansteuern(v_Ghostcar);         // programmierte Geschwindigkeit
                            if ((CU_Daten.KFR == 1) && (CU_Daten.FR == 0))
                                Motor_und_Bremse_ansteuern(0);                  // Motor aus, z.B. in Startphase
                        }
                    }
                    
                    break;

                case PROGRAMMIERDATENWORT: //.................................  Programmierdatenwort
                    Programmierdatenwort_decodieren();
                    if (CU_Prg_Daten.Adresse == eigene_Adresse)
                    {
                        if (                                                    // das wid immer 2 x bertragen
                                (CU_Prg_Daten.Parameter == PRG_MAX_GESCHWINDIGKEIT) ||
                                (CU_Prg_Daten.Parameter == PRG_BREMSWIRKUNG) //||
                                //(CU_Prg_Daten.Parameter == TANK_INHALT)
                            )
                        {
                            //. . . . . . . . . . . . . . . . . . . . . . . . .    max. Geschwindigkeit oder Bremswirkung
                            if (                                                // Wenn das Parametrierwort schon mal gesendet wurde
                                    (CU_Prg_Daten.Parameter == CU_Prg_Daten.Parameter_alt) &&
                                    (CU_Prg_Daten.Wert == CU_Prg_Daten.Wert_alt)
                               )

                            {
                                Parameter_verarbeiten();                        // max. Gewschindigkeit und Bremswirkung speichern
                                CU_Prg_Daten.Parameter_alt = PRG_NORMALBETRIEB;
                                CU_Prg_Daten.Wert_alt = 0;
                            }
                            else
                            {
                                CU_Prg_Daten.Parameter_alt = CU_Prg_Daten.Parameter;// merken was gerade gesendet wurde
                                CU_Prg_Daten.Wert_alt = CU_Prg_Daten.Wert;
                            }
                        }
                        else
                        {
                            if (CU_Prg_Daten.Parameter == PRG_NORMALBETRIEB)
                            {
                                if ((CU_Prg_Daten.Wert & 0x08) != 0)
                                    Tank_leer = JA;
                                else
                                    Tank_leer = NEIN;
                            }
                        }

                    }
                    break;
                                       
            }

            //----------------------------------------------------------------  ab hier alle ca. 100msec
            //if (Timer_Counter > 13)                                           // wenn alle Telegramme detektiert werden
            if (Timer_Counter > 10)                                             // wenn die Aktivdatenwrter nicht detektiert werden                            
            {
                Timer_Counter = 0;
                
                
                
                //..............................................................
                if (Adr_warte_auf_Reset_timeout > 0)                            // Programmiersequenz ist gestartet aber es ist noch kein Neustart gewesen
                {
                    Adr_warte_auf_Reset_timeout--;
                    if (Adr_warte_auf_Reset_timeout == 0)                       // wenn 2 Sekunden kein Neuart
                    {
                        Programmiermodus = NEIN;                                // dann Programmiermodus verlassen
                        EE_write_verified (NEIN, EE_ADR_PROGRAMMIERMODUS);
                    }
                }

                //..............................................................
                if (Adr_Programmier_timeout > 0)                                // Programmiermodus nach Neustart nach 2 Sekunden abbrechen
                {
                    Pin_o_LICHT ^= 1;
                    Adr_Programmier_timeout--;                                  // Timeout fr Adresssierung Auto Adresse
                    if (Adr_Programmier_timeout == 0)
                    {
                        Programmiermodus = NEIN;                                // dann Programmiermodus verlassen                        
                        EE_write_verified (NEIN, EE_ADR_PROGRAMMIERMODUS);
                        Licht_schalten();                                       // je nach Wert im EEPROM Licht ein oder aus
                    }
                }

                //..............................................................
                if (Ghostcar_Doppelklick_timeout > 0)                           // Zeitfenster fr Doppelklick nach Adr.-Programmierung
                    Ghostcar_Doppelklick_timeout--;                             // beenden

                
                //..............................................................
                if (Bremslicht_timer > 0)
                {
                    Bremslicht_timer--;
                    Pin_o_BREMSLICHT = 1;
                    CCPR4L = 0xFE;                                              // PWM fr Bremslicht voll auf
                }
                else
                {
                    Pin_o_BREMSLICHT = 0;                                       // Bremslicht aus 

                    if (Pin_o_LICHT == 1)                                       // wenn Licht an, dann Rcklicht glimmen lassen 
                        #ifdef KOMBI_LEUCHTEN
                        $$$CCPR4L = 0x10;
                        #endif

                        #ifdef SEPARATE_LEUCHTEN
                        CCPR4L = 0x00;
                        #endif
                    else
                        CCPR4L = 0x00;                                          // sonst Rcklicht ganz aus
                }
               
                //..............................................................
                for (Adresse = 0; Adresse < MAX_REGLER; Adresse++)              // fr alle Regler
                {
                    if (Spurwechseltaste.Reset_Timer[Adresse] > 0)              // wenn 2 Sekunden keine Taste gedrckt, dann
                    {                                                           // den Counter lschen
                        Spurwechseltaste.Reset_Timer[Adresse]--;                // d.h. zum Programmieren muss die Taste 2 x innerhalb
                        if (Spurwechseltaste.Reset_Timer[Adresse] == 0)         // von 2 Sekunden gedrckt werden
                        {
                            Spurwechseltaste.Counter[Adresse] = 0;
                        }
                    }
                }


                //..............................................................
                
                Ghostcar_Programmiermodus = EE_read(EE_ADR_GHOST_PROGRAMMIERMODUS);
                if (
                        (Ghostcar_Programmiermodus == JA) ||
                        (Tank_leer_Blinken == JA)
                    )
                                                                                // Im Ghostcar Programmiermodus (Geschwindigkeit einstellen)
                                                                                // und wenn der Tank leer ist, dann
                {                                                               // langsam blinken
                    Pin_o_LICHT ^= 1;  
                    
                    if ((Tank_leer_Blinken == JA) && (Tank_leer == NEIN))       // wenn Tank wieder voll, dann Licht schalten wie es war
                    {
                        Tank_leer_Blinken = NEIN;                               // aufhren mit Blinken
                        Licht_schalten();
                    }
                }
            } // Ende von 131 ms vorbei
        } // Ende von es wurde ein Telegramm empfangen
    } // Ende Hauptschleife
}	





